using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using SdlDotNet;

using Physics.Entities;
using Physics.Entities.Physics;
using Physics.Entities.Particles;

using Physics.Config.Files;

namespace Physics
{
    static class Debugger
    {
        static SdlDotNet.Font fpsFont = new SdlDotNet.Font("FreeSans.ttf", 18);

        static bool fpsOn = false;
        static bool collInfoOn = false;
        static bool showGridsOn = false;

        static List<LineP> gridLines = new List<LineP>();

        static Debugger_cfg cfg;

        public static void Initalize()
        {
            CreateGridLines();
            cfg = new Debugger_cfg();
        }

        public static void HandleInput()
        {
            if (cfg.fps.wasPressed())
                fpsOn = !fpsOn;
            if (cfg.collInfo.wasPressed())
                collInfoOn = !collInfoOn;

            if (cfg.clearParticles.wasPressed())
                EntityManager.ClearParticles();

            if (cfg.showGrids.wasPressed())
            {
                if (showGridsOn)
                {
                    foreach (LineP l in gridLines)
                        EntityManager.Remove(l);
                }
                else
                {
                    foreach (LineP l in gridLines)
                        EntityManager.Add(l);
                }
                showGridsOn = !showGridsOn;
            }
            if (cfg.oldColl.wasPressed())
            {
                PhysicsUpdater.old = !PhysicsUpdater.old;
                if (PhysicsUpdater.old)
                    Console.WriteLine("Collisions set to the Old collision checks");
                else
                    Console.WriteLine("Collisions set to the New collision checks");
            }
            if (cfg.futureColl.wasPressed())
            {
                CollisionHandler.futureCollisions = !CollisionHandler.futureCollisions;
                if (CollisionHandler.futureCollisions)
                    Console.WriteLine("Collisions set to check future collisions");
                else
                    Console.WriteLine("Collisions set to check current collisions");
            }
        }

        public static void Draw(Surface target)
        {
            DrawTextScreen(target);
        }

        // draws numbers to the target surface that display the FPS and UPS of the last second
        static void DrawTextScreen(Surface target)
        {
            int top = 0;

            if (fpsOn)
            {
                Surface temp = fpsFont.Render("FPS: " + Game.getFPS + "   UPS: " + Game.getUPS, Color.Gold, true);
                target.Blit(temp, new Point(5, top));
                top += 20;
                temp.Dispose();
            }

            if (collInfoOn)
            {
                Surface temp2 = fpsFont.Render("Collision Calls: " + CollisionHandler.CollisionChecks, Color.Black, true);
                Surface temp3 = fpsFont.Render("Valid Collisions: " + CollisionHandler.ValidCollisions, Color.Black, true);

                target.Blit(temp2, new Point(5, top));
                top += 20;
                target.Blit(temp3, new Point(5, top));
                top += 20;

                temp2.Dispose();
                temp3.Dispose();
            }
        }

        private static void CreateGridLines()
        {
            int xMax = Game.videoSize.X;
            int yMax = Game.videoSize.Y;

            int squareSize = PhysicsUpdater.squareSize;

            int xSize = xMax / squareSize + 1; // = 13
            int ySize = yMax / squareSize + 1; // = 10

            for (int x = 0; x < xSize; x++)
            {
                Vector2 from = new Vector2(x * squareSize, 0);
                Vector2 to = new Vector2(x * squareSize, yMax);

                LineP l = new LineP(from, to, false);
                l.MaxArrowLength = 0;
                gridLines.Add(l);
            }
            for (int y = 0; y < ySize; y++)
            {
                Vector2 from = new Vector2(0, y * squareSize);
                Vector2 to = new Vector2(xMax, y * squareSize);

                LineP l = new LineP(from, to, false);
                l.MaxArrowLength = 0;
                gridLines.Add(l);
            }
        }
    }
}
