using System;
using System.Drawing;
using System.Collections.Generic;
using SdlDotNet;

using Physics.Entities;
using Physics.Entities.Particles;
using Physics.Entities.Physics;

using Physics.Config.Files;

namespace Physics
{
    static class EffectsManager
    {
        private static List<Effect> Effects = new List<Effect>();

        private static MouseWind Wind;
        private static bool WindIsOn = true;

        private static Gravity WorldGravity;
        private static bool GravityIsOn = true;

        private static GravityWell well;

        private static MouseAccel mForce;

        private static VelocityDraw veloDraw;
        private static bool DrawVelocity = false;

        private static Effects_cfg cfg;


        public static void Add(Effect E)
        {
            Effects.Add(E);
        }
        public static void Remove(Effect E)
        {
            Effects.Remove(E);
        }

        public static void Initalize()
        {
            cfg = new Effects_cfg();

            Wind = new MouseWind();
            WorldGravity = new Gravity();
            veloDraw = new VelocityDraw();

            if (WindIsOn)
                Add(Wind);
            if (GravityIsOn)
                Add(WorldGravity);
            if (DrawVelocity)
                Add(veloDraw);
        }

        public static void Apply(Physics.Entities.Entity E)
        {
            foreach (Effect Eff in Effects)
                Eff.ApplyEffect(E);
        }

        public static void Input()
        {
            GravInput();
            GravWellInput();
            DrawVeloInput();

            MouseForceInput();
            MouseWindInput();

            AntiAliasingInput();
        }

        private static void GravInput()
        {
            if (cfg.gravity.wasPressed())
            {
                if (GravityIsOn)
                {
                    Remove(WorldGravity);
                    Console.WriteLine("Gravity turned Off");
                }
                else
                {
                    Add(WorldGravity);
                    Console.WriteLine("Gravity turned On");
                }
                GravityIsOn = !GravityIsOn;
            }
        }
        private static void GravWellInput()
        {
            if (cfg.gravWell.isPressed())
            {
                if(well != null)
                    Remove(well);

                well = new GravityWell(Mouse.MousePosition);
                Add(well);
            }
            else if(well != null)
            {
                Remove(well);
                well = null;
            }
        }
        private static void DrawVeloInput()
        {
            if (cfg.drawVelocity.wasPressed())
            {
                if (DrawVelocity)
                    Remove(veloDraw);
                else
                    Add(veloDraw);

                DrawVelocity = !DrawVelocity;
            }
        }

        private static bool wasForceLocked;
        private static void MouseForceInput()
        {
            if (cfg.mouseForce.wasPressed())
            {
                Remove(mForce); // removes the current force, is null OR a locked force
                mForce = new MouseAccel(new Vector2(Mouse.MousePosition));
                Add(mForce);
            }
            if (cfg.mouseForce.isPressed())
            {
                if (!wasForceLocked)
                {
                    if (InputMonitor.isPressed(Key.Return))
                        wasForceLocked = true;

                    Vector2 mouseFrom = mForce.fromPosition;
                    Vector2 mouseAt = new Vector2(Mouse.MousePosition);
                    mForce.UpdateTo(mouseAt);


                    LineP l = new LineP(mouseFrom, mouseAt, true);
                    l.color = Color.Brown;
                    EntityManager.Add(l);
                }
            }
            else if (cfg.mouseForce.wasReleased())
            {
                if(!wasForceLocked)
                    Remove(mForce);
                wasForceLocked = false;
            }
        }

        private static bool wasWindLocked;
        private static void MouseWindInput()
        {
            if (cfg.airFriction.wasPressed())
            {
                WindIsOn = !WindIsOn;

                if (WindIsOn)
                {
                    Console.WriteLine("AirFriction turned On");
                    Add(Wind);
                }
                else
                {
                    Console.WriteLine("AirFriction turned Off");
                    Remove(Wind);
                }
            }

            if (cfg.mouseWind.wasPressed())
            {
                Remove(Wind);
                Wind = new MouseWind(new Vector2(Mouse.MousePosition));

                if(WindIsOn)
                    Add(Wind);
            }
            if (cfg.mouseWind.isPressed())
            {
                if (!wasWindLocked)
                {
                    if (InputMonitor.isPressed(Key.Return))
                        wasWindLocked = true;

                    Vector2 mouseFrom = Wind.fromPosition;
                    Vector2 mouseAt = new Vector2(Mouse.MousePosition);
                    Wind.UpdateTo(mouseAt);

                    LineP l = new LineP(mouseFrom, mouseAt, true);
                    l.color = Color.DarkSeaGreen;
                    EntityManager.Add(l);
                }
            }
            else if (cfg.mouseWind.wasReleased())
            {
                if (!wasWindLocked)
                    Wind.Zero();
                wasWindLocked = false;
            }

        }

        private static void AntiAliasingInput()
        {
            if (cfg.antiAliasing.wasPressed())
                PhysicsE.antiAliasingOn = !PhysicsE.antiAliasingOn;
        }
    }
}
