using System;
using System.Collections.Generic;
using SdlDotNet;

using Physics.Entities.Physics;
using Physics.Entities.Particles;

namespace Physics.Entities
{
    static class EntityManager
    {
        // lists of Particles that will be managed
        private static List<ParticleE> ParticlesToAdd = new List<ParticleE>();
        private static List<ParticleE> ActiveParticles = new List<ParticleE>();
        private static List<ParticleE> ParticlesToRemove = new List<ParticleE>();

        // lists of PhysicsE that will be managed
        private static List<PhysicsE> PhysicsEToAdd = new List<PhysicsE>();
        private static List<PhysicsE> ActivePhysicsE = new List<PhysicsE>();
        private static List<PhysicsE> PhysicsEToRemove = new List<PhysicsE>();

        private static List<Player> PlayersToAdd = new List<Player>();
        private static List<Player> ActivePlayers = new List<Player>();
        private static List<Player> PlayersToRemove = new List<Player>();


        public static void Add(Entity E)
        {
            if (E is Player)
                PlayersToAdd.Add((Player)E);

            else if (E is ParticleE)
                ParticlesToAdd.Add((ParticleE)E);
            else if (E is PhysicsE)
                PhysicsEToAdd.Add((PhysicsE)E);
        }
        public static void Remove(Entity E)
        {
            if (E is Player)
                PlayersToRemove.Add((Player)E);

            else if (E is ParticleE)
                ParticlesToRemove.Add((ParticleE)E);
            else if (E is PhysicsE)
                PhysicsEToRemove.Add((PhysicsE)E);
        }

        // draws every active Entity to the screen
        public static void Draw(Surface target)
        {
            foreach (PhysicsE E in ActivePhysicsE)
                E.Draw(target);

            foreach (Player E in ActivePlayers)
                E.Draw(target);

            foreach (ParticleE E in ActiveParticles)
                E.Draw(target);
        }

        // runs every Entitys HandleInput method
        public static void HandleInput()
        {
            EffectsManager.Input();

            foreach (Player E in ActivePlayers)
                E.Input();
        }

        /* adds and removes Entities from the ActiveList,
         * runs the HandleCollisions method,
         * then runs every Entity's Update method,
         */
        public static void Update(float timeStep)
        {
            UpdatePhysics(timeStep);
            UpdateParticles(timeStep);
        }

        private static void UpdatePhysics(float timeStep)
        {
            // add the new Players to Active
            foreach (Player E in PlayersToAdd)
                ActivePlayers.Add(E);
            PlayersToAdd.Clear();

            // remove dead Players from Active
            foreach (Player E in PlayersToRemove)
                ActivePlayers.Remove(E);
            PlayersToRemove.Clear();


            // add the new PhysicsE to Active
            foreach (PhysicsE E in PhysicsEToAdd)
                ActivePhysicsE.Add(E);
            PhysicsEToAdd.Clear();

            // remove dead PhysicsE from Active
            foreach (PhysicsE E in PhysicsEToRemove)
                ActivePhysicsE.Remove(E);
            PhysicsEToRemove.Clear();

            PhysicsUpdater.CreateLists(ActivePlayers, ActivePhysicsE);
            PhysicsUpdater.Update(timeStep);
        }
        private static void UpdateParticles(float timeStep)
        {
            // add the new Particles to Active
            foreach (ParticleE E in ParticlesToAdd)
                ActiveParticles.Add(E);
            ParticlesToAdd.Clear();

            // remove dead Particles from Remove
            // doing this before the checking ensures that each Particle is drawn at least once
            foreach (ParticleE E in ParticlesToRemove)
                ActiveParticles.Remove(E);
            ParticlesToRemove.Clear();

            // do an update on all active Particles
            foreach (ParticleE E in ActiveParticles)
            {
                EffectsManager.Apply(E);
                E.Update(timeStep);
            }

            // figure out which Particles are dead
            foreach (ParticleE E in ActiveParticles)
            {
                if (E.isDead)
                    ParticlesToRemove.Add(E);
            }
        }

        public static void ClearParticles()
        {
            ActiveParticles.Clear();
        }
    }
}
