using System;
using System.Drawing;
using SdlDotNet;

namespace Physics.Entities.Particles
{
    class LineP : ParticleE
    {
        private Triangle tri;
        private Line line;
        private Vector2 dest;

        public Color color = Color.Green;
        public float LengthScale = .5F;

        public float MinArrowLength = 0;
        public float MaxArrowLength = 20;

        public LineP(Vector2 fromVec, Vector2 toVec, bool canDie)
            : base(fromVec, canDie)
        {
            dest = toVec.Clone();
        }

        public LineP(Vector2 fromVec, Vector2 toVec, int lifeTime)
            : base(fromVec, lifeTime)
        {
            dest = toVec.Clone();
        }

        public override void Update(float timeStep)
        {
            base.Update(timeStep);

            if ((Position - dest).Length <= 0)
                return;

            float length = (Position - dest).Length * LengthScale;

            if (length > MaxArrowLength)
                length = MaxArrowLength;
            else if (length < MinArrowLength)
                length = MinArrowLength;

            Vector2 der = (Position - dest).Unit * length;
            Vector2 pi = der.Perp / 2;
            Vector2 t1 = dest + der + pi;
            Vector2 t2 = dest + der - pi;

            line = new Line(Position.Point, dest.Point);
            tri = new Triangle(dest.Point, t1.Point, t2.Point);
        }

        public override void Draw(Surface target)
        {
            // if (line != null)
			target.DrawLine(line, color);

            if (! tri.Point2.Equals(tri.Point3) ) // tri == null
               target.DrawFilledTriangle(tri, color);
        }
    }
}
