using System;
using System.Drawing;
using SdlDotNet;

using Physics.Entities.Particles;

namespace Physics.Entities.Physics
{
    // all collisions done without friction

    /// <summary>
    /// the collision math was taken from a few physics articles
    /// created by Chris Hecker.
    /// 
    /// i had my own ball-ball and ball-polygon collsions but i could not get poly-poly,
    /// i then used his and reworked alot of my code. 
    /// </summary>
    static class CollisionHandler
    {
        // excludes the screen checks
        public static int CollisionChecks = 0;
        public static int ValidCollisions = 0;

        public static bool futureCollisions = false;
        public static bool subDivideOnSpeed = false;

        public static void HandleCollisionAt(PhysicsE E1, PhysicsE E2, float timeStep)
        {
            if (E1 is Ball && E2 is Ball)
                HandleCollisionAt((Ball)E1, (Ball)E2, timeStep);

            if (E1 is Ball && E2 is EPolygon)
                HandleCollisionAt((Ball)E1, (EPolygon)E2, timeStep);
            if (E2 is Ball && E1 is EPolygon)
                HandleCollisionAt((Ball)E2, (EPolygon)E1, timeStep);

            if (E1 is EPolygon & E2 is EPolygon)
                HandleCollisionAt((EPolygon)E1, (EPolygon)E2, timeStep);
        }
        public static void HandleScreenCollisionAt(PhysicsE E, float timeStep)
        {
            if (E is Ball)
                HandleStaticCollisionAt((Ball)E, ScreenInfoAt((Ball)E, timeStep), timeStep);
            if (E is EPolygon)
                HandleStaticCollisionAt((EPolygon)E, ScreenInfoAt((EPolygon)E, timeStep), timeStep);
        }

        private static void HandleCollisionAt(Ball Ball1, Ball Ball2, float timeStep)
        {
            if (Ball1.StaticCollisions && Ball2.StaticCollisions
                && !(Ball1 is Player) && !(Ball2 is Player))
                return;

            CollisionInfo Info = GetInfoAt(Ball1, Ball2, timeStep);

            if (Info == null)
                return;
            ValidCollisions++;

            if (Ball1.StaticCollisions)
            {
                if (Ball1 is Player && Ball2.StaticCollisions)
                {
                    HandleStaticCollisionAt(Ball1, Info, timeStep);
                }
                else
                {
                    Info.Normal = (-1) * Info.Normal;
                    HandleStaticCollisionAt(Ball2, Info, timeStep);
                }
                return;
            }

            if (Ball2.StaticCollisions)
            {
                if (Ball2 is Player && Ball1.StaticCollisions)
                {
                    Info.Normal = (-1) * Info.Normal;
                    HandleStaticCollisionAt(Ball2, Info, timeStep);
                }
                else
                {
                    HandleStaticCollisionAt(Ball1, Info, timeStep);
                }
                return;
            }

            // a few values to simplify the equations
            float m1 = Ball1.Mass;
            float m2 = Ball2.Mass;

            float e = (Ball1.CoR + Ball2.CoR) / 2;
            Vector2 normal = Info.Normal;

            Vector2 V1 = Ball1.VelocityAt(timeStep);
            Vector2 V2 = Ball2.VelocityAt(timeStep);


            float iTop = -(e + 1) * (V1 - V2).Dot(normal);

            // otherwise the collision happened and we do the math
            // the below assumes an object cannot rotated, and collisions have no friction

            float impulse = iTop / (normal.Dot(normal) * (1 / m1 + 1 / m2));

            Ball1.ApplyImpulse(impulse / m1 * normal, Info.Point);
            Ball2.ApplyImpulse(-impulse / m2 * normal, Info.Point);

            if (Ball1.Rotates && Ball2.Rotates)
            {
                // the simple rotation math
                //float newR = (Ball1.RVelocity * Ball1.RMass + Ball2.RVelocity * Ball2.RMass) / (Ball1.RMass + Ball2.RMass);
                //Ball1.RVelocity = newR;
                //Ball2.RVelocity = -newR;
            }
        }
        private static void HandleCollisionAt(Ball Ball, EPolygon Poly, float timeStep)
        {
            if (Ball.StaticCollisions && Poly.StaticCollisions && !(Ball is Player))
                return;

            if (subDivideOnSpeed)
            {
                int num = (int)((Ball.Position - Ball.PositionAt(timeStep)).Length / Ball.Radius + 1);

                for (int i = 1; i <= num; i++)
                {
                    HandleStaticCollisionAt(Ball, GetInfoAt(Ball, Poly, timeStep), (timeStep * i) / num);
                }
            }
            else
                HandleStaticCollisionAt(Ball, GetInfoAt(Ball, Poly, timeStep), timeStep);
        }
        private static void HandleCollisionAt(EPolygon Poly1, EPolygon Poly2, float timeStep)
        {
            if (Poly1.StaticCollisions && Poly2.StaticCollisions)
                return;

            // CollisionInfo Info = GetInfo(Poly1, Poly2);
            // to do
        }

        private static CollisionInfo GetInfoAt(Ball Ball1, Ball Ball2, float timeStep)
        {
            CollisionChecks++;

            // a few values to simplify the equations
            float R1 = Ball1.Radius;
            float R2 = Ball2.Radius;

            Vector2 P1;
            Vector2 P2;
            Vector2 V1;
            Vector2 V2;
            if (futureCollisions)
            {
                P1 = Ball1.PositionAt(timeStep);
                P2 = Ball2.PositionAt(timeStep);
                V1 = Ball1.VelocityAt(timeStep);
                V2 = Ball2.VelocityAt(timeStep);
            }
            else
            {
                P1 = Ball1.Position;
                P2 = Ball2.Position;
                V1 = Ball1.Velocity;
                V2 = Ball2.Velocity;
            }

            // quick binding box check
            if (P1.X - R1 > P2.X + R2 || P1.X + R1 < P2.X - R2 ||
                P1.Y - R1 > P2.Y + R2 || P1.Y + R1 < P2.Y - R2)
                return null;

            // test if not touching
            if ((P1 - P2).Length >= R1 + R2)
                return null;

            // test if they are moving apart in some way
            // if they arn't it's likely that they collided last frame and are still overlapping
            if ((V1 - V2).Dot(P1 - P2) >= 0)
                return null;

            return new CollisionInfo(P1 - (P1 - P2) * R1 / (R1 + R2), P1 - P2);
        }
        private static CollisionInfo GetInfoAt(Ball Ball, EPolygon Poly, float timeStep)
        {
            CollisionChecks++;

            Vector2 Velo = Ball.VelocityAt(timeStep);
            Vector2 Pos = Ball.PositionAt(timeStep);
            float R = Ball.Radius;

            float maxX = Poly.maxX;
            float minX = Poly.minX;
            float maxY = Poly.maxY;
            float minY = Poly.minY;

            // quick binding box check
            if (Pos.X - R > maxX || Pos.X + R < minX ||
                Pos.Y - R > maxY || Pos.Y + R < minY)
                return null;


            Vector2[] pts = Poly.screenPoints;
            int num = pts.Length;

            Vector2 totVec = new Vector2(0, 0);
            int total = 0;

            for (int i = 0; i < num; i++)
            {
                Vector2 vec = Pos.VectorToLine(pts[i].X, pts[i].Y,
                                       pts[(i + 1) % num].X,
                                       pts[(i + 1) % num].Y);

                if (vec.Length <= Ball.Radius && Velo.Dot(vec) > 0)
                {
                    totVec += vec;
                    total += 1;
                }
            }

            if (total <= 0)
                return null;

            Vector2 collP = totVec / total + Pos;
            Vector2 norm = Pos - collP;

            return new CollisionInfo(collP, norm);
        }
        private static CollisionInfo GetInfoAt(EPolygon Poly1, EPolygon Poly2, float timeStep)
        {
            CollisionChecks++;

            // detect best collision point
            Vector2[] p1 = Poly1.screenPoints;
            Vector2[] p2 = Poly2.screenPoints;
            int n1 = p1.Length;
            int n2 = p2.Length;

            for (int x = 0; x < n1; x++)
            {
                for (int y = 0; y < n2; y++)
                {
                    Vector2 vec = Vector2.LineIntersection(p1[x], p1[(x + 1) % n1],
                                                           p2[y], p2[(y + 1) % n2]);
                    if (vec != null)
                    {
                        PointP p = new PointP(vec, 1000);
                        EntityManager.Add(p);
                    }
                }
            }


            return null;
        }

        // currently NOT used
        // uses the basic rotation model
        private static void HandleScreenCollision(Ball Ball)
        {
            int R = Ball.Radius;

            int minX = R;
            int minY = R;
            int maxX = Video.Screen.Width - R;
            int maxY = Video.Screen.Height - R;

            // check collisions with the edges of the screen
            // do very basic rotation math
            if (Ball.Position.X > maxX && Ball.Velocity.X > 0) // right
            {
                if (Ball.CoR < 1)
                { Ball.Position.X = maxX; }
                Ball.Velocity.X *= -Ball.CoR;
                //Ball.RVelocity = Ball.Velocity.Y / R;
            }
            if (Ball.Position.X < minX && Ball.Velocity.X < 0) // left
            {
                if (Ball.CoR < 1)
                { Ball.Position.X = minX; }
                Ball.Velocity.X *= -Ball.CoR;
                //Ball.RVelocity = -Ball.Velocity.Y / R;
            }

            if (Ball.Position.Y > maxY && Ball.Velocity.Y > 0) // bottom
            {
                if (Ball.CoR < 1)
                { Ball.Position.Y = maxY; }
                Ball.Velocity.Y *= -Ball.CoR;
                //Ball.RVelocity = -Ball.Velocity.X / R;
            }
            if (Ball.Position.Y < minY && Ball.Velocity.Y < 0) // top
            {
                if (Ball.CoR < 1)
                { Ball.Position.Y = minY; }
                Ball.Velocity.Y *= -Ball.CoR;
                //Ball.RVelocity = Ball.Velocity.X / R;
            }
        }

        // does not cause rotation
        // causes object to penetrate with low CoR
        private static void HandleStaticCollisionAt(Ball Ball, CollisionInfo Info, float timeStep)
        {
            if (Info == null)
                return;
            ValidCollisions++;

            float e = Ball.CoR;
            Vector2 normal = Info.Normal;

            Vector2 Velo = Ball.VelocityAt(timeStep);

            float iTop = -(e + 1) * Velo.Dot(normal);

            // impulse div by mass
            float idm = iTop / (normal.Dot(normal));

            Vector2 rotImpulse = new Vector2(0, 0);
            if (Ball.Rotates)
                rotImpulse = Velo.Projection(normal) - Velo;

            float maxF = .1F;
            if (rotImpulse.Length > maxF)
                rotImpulse = rotImpulse.Unit * maxF;

            Ball.ApplyImpulse(idm * normal + rotImpulse, Info.Point);

            // from center to point
            Vector2 CollP = normal / normal.Length * Ball.Radius;
            Ball.Position = Info.Point + CollP; // CoR penetration fix, adds the jitters
        }
        private static void HandleStaticCollisionAt(EPolygon Poly, CollisionInfo Info, float timeStep)
        {
            if (Info == null)
                return;
            ValidCollisions++;

            PointP p = new PointP(Info.Point, 1000);
            EntityManager.Add(p);


            Vector2 Pos = Poly.PositionAt(timeStep);
            Vector2 Velo = Poly.VelocityAt(timeStep);

            float m1 = Poly.Mass;

            float e = Poly.CoR;
            Vector2 normal = Info.Normal;

            // from center of mass to collision points
            Vector2 Ra = Info.Point - Pos;
            Vector2 Vab = Velo + Poly.RVelocity * Ra;

            float i1 = -(1 + e) * Vab.Dot(normal);
            float i2 = normal.Dot(normal) * (1 / m1);
            float i3 = (Ra.Dot(normal)) * (Ra.Dot(normal)) / Poly.RMass;
            float impulse = i1 / (i2 + i3);

            Poly.Velocity += (impulse * normal) / m1;
            //Poly.RVelocity += Ra.Dot(impulse * normal) / Poly.RMass;
        }

        private static CollisionInfo ScreenInfoAt(Ball Ball, float timeStep)
        {
            CollisionChecks++;

            int R = Ball.Radius;

            int minX = R;
            int minY = R;
            int maxX = Video.Screen.Width - R;
            int maxY = Video.Screen.Height - R;

            Vector2 Pos = Ball.Position;
            Vector2 Velo = Ball.Velocity;

            // check collisions with the edges of the screen
            if (Pos.X > maxX && Velo.X > 0) // right
                return new CollisionInfo(
                    new Vector2(Video.Screen.Width, Pos.Y),
                    new Vector2(-1, 0));
            if (Pos.X < minX && Velo.X < 0) // left
                return new CollisionInfo(
                    new Vector2(0, Pos.Y),
                    new Vector2(1, 0));

            if (Pos.Y > maxY && Velo.Y > 0) // bottom
                return new CollisionInfo(
                    new Vector2(Pos.X, Video.Screen.Height),
                    new Vector2(0, -1));
            if (Pos.Y < minY && Velo.Y < 0) // top
                return new CollisionInfo(
                    new Vector2(Pos.X, 0),
                    new Vector2(0, 1));

            return null;
        }
        private static CollisionInfo ScreenInfoAt(EPolygon Poly, float timeStep)
        {
            CollisionChecks++;

            Vector2[] pts = Poly.screenPoints;
            int num = pts.Length;

            for (int i = 0; i < num; i++)
            {
                if (pts[i].X < 0) // left
                    return new CollisionInfo(pts[i], new Vector2(1, 0));
                else if (pts[i].X > Video.Screen.Width) // right
                    return new CollisionInfo(pts[i], new Vector2(-1, 0));
                else if (pts[i].Y < 0) // up
                    return new CollisionInfo(pts[i], new Vector2(0, 1));
                else if (pts[i].Y > Video.Screen.Height) // down
                    return new CollisionInfo(pts[i], new Vector2(0, -1));
            }

            return null;
        }

        // a method that 'should' work for non-static polygon-polygon collisions ... ?
        private static void Test(EPolygon Poly1, EPolygon Poly2)
        {
            Vector2 Pa = Poly1.Position;
            Vector2 Pb = Poly2.Position;
            Vector2 Va = Poly1.Velocity;
            Vector2 Vb = Poly2.Velocity;

            float m1 = Poly1.Mass;
            float m2 = Poly2.Mass;

            float e = (Poly1.CoR + Poly2.CoR) / 2;
            Vector2 normal = Poly1.Position - Poly2.Position;

            // from center of mass to collision points
            Vector2 Ra = -1 / 2 * normal;
            Vector2 Rb = 1 / 2 * normal;
            Vector2 Vab = Va + Poly1.RVelocity * Ra - (Vb + Poly2.RVelocity * Rb);

            float i1 = -(1 + e) * Vab.Dot(normal);
            float i2 = normal.Dot(normal) * (1 / m1 + 1 / m2);
            float i3 = (Ra.Dot(normal)) * (Ra.Dot(normal)) / Poly1.RMass;
            float i4 = (Rb.Dot(normal)) * (Rb.Dot(normal)) / Poly2.RMass;
            float impulse = i1 / (i2 + i3 + i4);

            Poly1.Velocity += (impulse * normal) / m1;
            Poly2.Velocity += (-impulse * normal) / m2;

            //Poly1.RVelocity += Ra.Dot(impulse * normal) / Poly1.RMass;
            //Poly2.RVelocity += Rb.Dot(-impulse * normal) / Poly2.RMass;
        }
    }

    class CollisionInfo
    {
        public Vector2 Point, Normal;

        public CollisionInfo(Vector2 point, Vector2 normal)
        {
            Point = point;
            Normal = normal;
        }
    }
}
