using System;
using System.Collections.Generic;
using System.Text;

namespace Physics.Entities.Physics
{
    static class PhysicsUpdater
    {
        public static int squareSize = 32;

        private static List<Ball> Balls;
        private static List<EPolygon> Polys;

        private static List<Player> playersOld;
        private static List<PhysicsE> physicsEOld;
        public static bool old = false;

        public static void CreateLists(List<Player> players, List<PhysicsE> physicsE)
        {
            if (old)
            {
                playersOld = players;
                physicsEOld = physicsE;
                return;
            }

            Balls = new List<Ball>();
            Polys = new List<EPolygon>();

            foreach (Player P in players)
                Balls.Add((Ball)P);

            foreach (PhysicsE PE in physicsE)
            {
                if (PE is Ball)
                    Balls.Add((Ball)PE);
                else
                    Polys.Add((EPolygon)PE);
            }
        }

        public static void Update(float timeStep)
        {
            if (old)
            {
                foreach (Player P in playersOld)
                    EffectsManager.Apply(P);
                foreach (PhysicsE PE in physicsEOld)
                    EffectsManager.Apply(PE);

                HandleOld(0);

                // do an update on all active Entities
                foreach (Player P in playersOld)
                    P.Update(timeStep);
                foreach (PhysicsE PE in physicsEOld)
                    PE.Update(timeStep);
            }
            else
            {
                foreach (Ball B in Balls)
                    EffectsManager.Apply(B);
                foreach (EPolygon P in Polys)
                    EffectsManager.Apply(P);

                // checks Collisions with each pair of PhysicsE
                // this MAY belong AFTER the update all
                HandleCollisions(timeStep);

                // do an update on all active Entities
                foreach (Ball B in Balls)
                    B.Update(timeStep);
                foreach (EPolygon P in Polys)
                    P.Update(timeStep);
            }
        }

        // Handles Collisions between all Entities
        private static void HandleCollisions(float timeStep)
        {
            List<PhysicsE>[,] Array = CreateArray();

            foreach (List<PhysicsE> Blist in Array)
            {
                for (int x = 0; x < Blist.Count; x++)
                {
                    PhysicsE E1 = Blist[x];

                    for (int y = x + 1; y < Blist.Count; y++)
                    {
                        PhysicsE E2 = Blist[y];
                        CollisionHandler.HandleCollisionAt(E1, E2, timeStep);
                    }
                }
            }

            //foreach (EPolygon P in Polys)
            //    CollisionHandler.HandleScreenCollision(P);

            foreach (Ball B in Balls)
                CollisionHandler.HandleScreenCollisionAt(B, timeStep);

            //foreach (EPolygon P in Polys)
            //{
            //    foreach (Ball B in Balls)
            //        CollisionHandler.HandleCollisionAt(B, P, timeStep);
            //}
        }

        private static void HandleOld(float timeStep)
        {
            for (int x = 0; x < physicsEOld.Count; x++)
            {
                PhysicsE E1 = physicsEOld[x];
                for (int y = x + 1; y < physicsEOld.Count; y++)
                {
                    PhysicsE E2 = physicsEOld[y];
                    CollisionHandler.HandleCollisionAt(E1, E2, timeStep);
                }
                CollisionHandler.HandleScreenCollisionAt(E1, timeStep);
            }

            foreach (Player P in playersOld)
            {
                foreach (PhysicsE PE in physicsEOld)
                    CollisionHandler.HandleCollisionAt(P, PE, timeStep);
                CollisionHandler.HandleScreenCollisionAt(P, timeStep);
            }
        }

        private static List<PhysicsE>[,] CreateArray()
        {
            int xMax = Game.videoSize.X;
            int yMax = Game.videoSize.Y;

            int xSize = xMax / squareSize + 1; // = 13
            int ySize = yMax / squareSize + 1; // = 10

            List<PhysicsE>[,] a = new List<PhysicsE>[xSize, ySize];
            for (int x = 0; x < xSize; x++)
            {
                for (int y = 0; y < ySize; y++)
                {
                    a[x, y] = new List<PhysicsE>();
                }
            }

            foreach (Ball B in Balls)
            {
                int left = (int)(B.Position.X - B.Radius) / squareSize;
                int right = (int)(B.Position.X + B.Radius) / squareSize;
                int top = (int)(B.Position.Y - B.Radius) / squareSize;
                int bottom = (int)(B.Position.Y + B.Radius) / squareSize;

                if (left < 0)
                    left = 0;
                if (right > xSize - 1)
                    right = xSize - 1;
                if (top < 0)
                    top = 0;
                if (bottom > ySize - 1)
                    bottom = ySize - 1;

                for (int x = left; x <= right; x++)
                {
                    for (int y = top; y <= bottom; y++)
                        a[x, y].Add(B);
                }
            }
            foreach (EPolygon P in Polys)
            {
                int left = (int)(P.minX) / squareSize;
                int right = (int)(P.maxX) / squareSize;
                int top = (int)(P.minY) / squareSize;
                int bottom = (int)(P.maxY) / squareSize;

                if (left < 0)
                    left = 0;
                if (right > xSize - 1)
                    right = xSize - 1;
                if (top < 0)
                    top = 0;
                if (bottom > ySize - 1)
                    bottom = ySize - 1;

                for (int x = left; x <= right; x++)
                {
                    for (int y = top; y <= bottom; y++)
                        a[x, y].Add(P);
                }
            }
            return a;
        }
    }
}
