using System;
using System.Collections.Generic;
using SdlDotNet;

namespace Physics
{
    static class InputMonitor
    {
        static State[] KeyStates = new State[323];
        static State[] ButtonStates = new State[5];

        static Stack<State[]> KeyStack = new Stack<State[]>();
        static Stack<State[]> ButtonStack = new Stack<State[]>();

        public static void UpdateStates()
        {
            for (int i = 0; i < KeyStates.Length; i++)
            {
                State state = KeyStates[i];

                if (Keyboard.IsKeyPressed((Key) i))
                {
                    if (state.Equals(State.isReleased) || state.Equals(State.wasReleased))
                        KeyStates[i] = State.wasPressed;
                    else
                        KeyStates[i] = State.isPressed;
                }
                else
                {
                    if (state.Equals(State.isPressed) || state.Equals(State.wasPressed))
                        KeyStates[i] =  State.wasReleased;
                    else
                        KeyStates[i] =  State.isReleased;
                }
            }

            for (int i = 0; i < ButtonStates.Length; i++)
            {
                State state = ButtonStates[i];

                if (Mouse.IsButtonPressed((MouseButton)i))
                {
                    if (state.Equals(State.isReleased) || state.Equals(State.wasReleased))
                        ButtonStates[i] = State.wasPressed;
                    else
                        ButtonStates[i] = State.isPressed;
                }
                else
                {
                    if (state.Equals(State.isPressed) || state.Equals(State.wasPressed))
                        ButtonStates[i] = State.wasReleased;
                    else
                        ButtonStates[i] = State.isReleased;
                }
            }
        }

        public static void Push()
        {
            KeyStack.Push((State[])KeyStates.Clone());
            ButtonStack.Push((State[])ButtonStates.Clone());
        }
        public static void Pop()
        {
            KeyStates = KeyStack.Pop();
            ButtonStates = ButtonStack.Pop();
        }

        public static bool wasPressed(Key key)
        {
            State state = KeyStates[(int)key];
            return state.Equals(State.wasPressed);
        }
        public static bool isPressed(Key key)
        {
            State state = KeyStates[(int)key];
            return state.Equals(State.isPressed) || state.Equals(State.wasPressed);
        }
        public static bool wasReleased(Key key)
        {
            State state = KeyStates[(int)key];
            return state.Equals(State.wasReleased);
        }
        public static bool isReleased(Key key)
        {
            State state = KeyStates[(int)key];
            return state.Equals(State.isReleased) || state.Equals(State.wasReleased);
        }

        public static bool wasPressed(MouseButton button)
        {
            State state = ButtonStates[(int)button];
            return state.Equals(State.wasPressed);
        }
        public static bool isPressed(MouseButton button)
        {
            State state = ButtonStates[(int)button];
            return state.Equals(State.isPressed) || state.Equals(State.wasPressed);
        }
        public static bool wasReleased(MouseButton button)
        {
            State state = ButtonStates[(int)button];
            return state.Equals(State.wasReleased);
        }
        public static bool isReleased(MouseButton button)
        {
            State state = ButtonStates[(int)button];
            return state.Equals(State.isReleased) || state.Equals(State.wasReleased);
        }

    }

    enum State
    {
        isReleased,
        wasPressed,
        isPressed,
        wasReleased,
    }
}
